@@ -1,27 +1,22 @@
 # ABSTRACT: turn WGS84 coordinates into three word addresses and vice-versa using what3words.com HTTPS API
 
-
 package Geo::What3Words;
-$Geo::What3Words::VERSION = '2.1.2';
+$Geo::What3Words::VERSION = '2.1.3';
 use strict;
 use warnings;
-use URI;
+use URI::XS;
 use LWP::UserAgent;
-use LWP::Protocol::https;
+#use LWP::Protocol::https;
 use Cpanel::JSON::XS;
 use Data::Dumper;
+use Encode qw( decode_utf8 );
 use Net::Ping;
 use Net::Ping::External;
-use Encode;
+use Ref::Util qw( is_hashref is_coderef );
 my $JSONXS = Cpanel::JSON::XS->new->allow_nonref(1);
 
 
 
-
-
-
-
-
 sub new {
   my ($class, %params) = @_;
 
@@ -42,17 +37,12 @@ sub new {
 }
 
 
-
-
-
-
-
 sub ping {
   my $self = shift;
 
   ## http://example.com/some/path => example.com
   ## also works with IP addresses
-  my $host = URI->new($self->{api_endpoint})->host;
+  my $host = URI::XS->new($self->{api_endpoint})->host;
 
   $self->_log("pinging $host...");
 
@@ -65,47 +55,28 @@ sub ping {
 }
 
 
-
-
-
-
-
-
 sub words2pos {
   my ($self, @params) = @_;
-  my $res = $self->words_to_position(@params);
 
-  if ( $res && ref($res) eq 'HASH' && exists($res->{geometry}) ){
-    return $res->{geometry}->{lat} . ',' . $res->{geometry}->{lng};
+  my $res = $self->words_to_position(@params);
+  if ( $res && is_hashref($res) && exists($res->{geometry}) ){
+      return $res->{geometry}->{lat} . ',' . $res->{geometry}->{lng};
   }
   return;
 }
 
 
 
-
-
-
-
-
 sub pos2words {
   my ($self, @params) = @_;
   my $res = $self->position_to_words(@params);
-
-  if ( $res && ref($res) eq 'HASH' && exists($res->{words}) ){
-    return $res->{words};
+  if ( $res && is_hashref($res) && exists($res->{words}) ){
+      return $res->{words};
   }
   return;
 }
 
 
-
-
-
-
-
-
-
 sub valid_words_format {
   my $self = shift;
   my $words = shift;
@@ -120,10 +91,6 @@ sub valid_words_format {
 }
 
 
-
-
-
-
 sub words_to_position {
   my $self = shift;
   my $words = shift;
@@ -133,15 +100,6 @@ sub words_to_position {
 }
 
 
-
-
-
-
-
-
-
-
-
 sub position_to_words {
   my $self = shift;
   my $position = shift;
@@ -154,7 +112,6 @@ sub position_to_words {
 sub get_languages {
   my $self = shift;
   my $position = shift;
-
   return $self->_query_remote_api('languages');
 }
 
@@ -169,17 +126,18 @@ sub _query_remote_api {
   my $rh_params   = shift || {};
 
   my $rh_fields = {
-    a=> 1,
+      a      => 1,
       key    => $self->{key},
       format => 'json',
       %$rh_params
   };
+
   foreach my $key (keys %$rh_fields){
-    delete $rh_fields->{$key} if (!defined($rh_fields->{$key}));
+      delete $rh_fields->{$key} if (!defined($rh_fields->{$key}));
   }
 
-  my $uri = URI->new($self->{api_endpoint} . $method_name);
-  $uri->query_form( $rh_fields );
+  my $uri = URI::XS->new($self->{api_endpoint} . $method_name);
+  $uri->query( $rh_fields );
   my $url = $uri->as_string;
 
   $self->_log("GET $url");
@@ -203,12 +161,12 @@ sub _log {
   my $message = shift;
   return unless $self->{logging};
 
-  if ( ref($self->{logging}) eq 'CODE' ){
-    my $lc = $self->{logging};
-    &$lc("Geo::What3Words -- " . $message);
+  if ( is_coderef($self->{logging}) ){
+      my $lc = $self->{logging};
+      &$lc("Geo::What3Words -- " . $message);
   }
   else {
-    print "Geo::What3Words -- " . $message . "\n";
+      print "Geo::What3Words -- " . $message . "\n";
   }
   return;
 }
@@ -228,7 +186,7 @@ Geo::What3Words - turn WGS84 coordinates into three word addresses and vice-vers
 
 =head1 VERSION
 
-version 2.1.2
+version 2.1.3
 
 =head1 SYNOPSIS
 
@@ -253,7 +211,7 @@ This module calls API version 2 (https://docs.what3words.com/api/v2/) to convert
 coordinates into those 3 word addresses (forward) and 3 words into coordinates
 (reverse).
 
-Version 1 is deprecated and will stop working December 2016.
+Version 1 is deprecated and no longer works
 
 You need to sign up at http://what3words.com/login and then register for an API key
 at https://what3words.com/get-api-key/
@@ -426,7 +384,7 @@ mtmail <mtmail-cpan@gmx.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019 by OpenCage Data Limited.
+This software is copyright (c) 2019 by OpenCage GmbH.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
